import re, json
from smsgateway.sources.sms import command_list
from smsgateway.config import *
from smsgateway.sources.utils import *
from smsgateway import sink_sms
from telethon import TelegramClient, utils

app_log = setup_logging("telegram-send")

IDENTIFIER = "TG"

command_regex = re.compile('^(?P<command>[a-zA-Z ]+)$')

api_id = 242101
api_hash = "80cbc97ce425aae38c1e0291ef2ab2a4"

session_path = os.path.join(CONFIG_DIR, 'telegram-send')

def check(cmd, multiline):
    # print("Checking %s" % cmd)
    if cmd.lower() == IDENTIFIER.lower() and multiline:
      return True
    else:
      return False


def run(lines):
    print("Forwarding Telegram Message")
    messageStarted = False
    to_matched = None
    message = ""

    for line in lines[1:]: # skip IDENTIFIER
        if messageStarted:
            message += f"\n{line}"
        elif not line.strip(): # empty line
            messageStarted = True
            message += line
        else:
            mTo = re.match("To: (.*)$", line)
            if mTo:
                to_matched = mTo.group(1).strip()
            else:
                app_log.warning(f"Unkown header: {line}!")

    if to_matched and message:
      print("Starting client..")
      client = TelegramClient(session_path, api_id, api_hash)
      try:
          client.start()
      except Exception as e:
          ret = "Could not connect! Run python3 -m smsgateway.sources.commands.send_telegram_new to authorize!\nError: %s" % e
          app_log.error(ret)
          return ret

      to = None
      for x in client.iter_dialogs():
          name = utils.get_display_name(x.entity)
          if name == to_matched:
            to = x.entity.id
            print("Found it via display_name: %s" % x.entity.stringify())
            break
      if not to:
          print(f"Couldn't find {to}! Trying directly..")
          to = name = to_matched

      print("Sending Telegram msg:\n%s" % message)
      # arg = "msg %s %s" % (to, msg)
      # print("Calling Telegram with %s" % arg)
      import getpass
      print("I am: %s" % getpass.getuser())

      client.send_message(to, message)
      client.disconnect()
      ret = '\n'.join([
        IDENTIFIER,
        f"To: {name}",
        "",
        message
      ])
      # try:
      #     #res = subprocess.check_output(["sudo", "-Hu", "pi", "/home/pi/tg/bin/telegram-cli", "-D", "--json", "-RW", "-e", arg], stderr=subprocess.STDOUT).decode('UTF-8').strip()
      #     res = subprocess.check_output([TELEGRAM_CLI_PATH, "-D", "--json", "-k", TELEGRAM_KEY_PATH, "--profile", "profile_send", "-RW", "-e", arg], stderr=subprocess.STDOUT, timeout=300).decode('UTF-8').strip()
      # except subprocess.TimeoutExpired as e:
      #     return "Timeout when sending TG to %s: %s" % (to, e)
      # except subprocess.CalledProcessError as e:
      #     return "Failed to send TG to %s: %s" % (to, e.output)
      # lines = res.split('\n')
      # print("Got %s" % lines)
      # if len(lines) >= 1:
      #   for l in lines:
      #       try:
      #           j = json.loads(l.strip())
      #           if j and j['result'] == 'SUCCESS':
      #             ret = f'TG\nTo: {to}\n{msg}'
      #             break
      #           elif j:
      #             try:
      #                 ret = f"Failed to send TG to {to}:\n{j['error']}"
      #                 break
      #             except Exception as e:
      #                 ret = "Failed to send TG to %s!" % to
      #           else:
      #             ret = "Failed to send TG to %s!" % to
      #       except ValueError:
      #           ret = "Failed to send TG to %s!" % to
      # else:
      #   ret = "Failed to send TG to %s!" % to
    else:
      ret = f"Couldn't match To: {to_matched} or message {message}"
      print(ret)
    return ret

if __name__ == '__main__':
    client = TelegramClient(session_path, api_id, api_hash)
    if not client.start():
        app_log.error("Could not connect! Run python3 -m smsgateway.source.telegram to authorize!")
        sys.exit(1)

command_list.append({
    'name' : 'TG-Forwarder-new',
    'check': check,
    'run': run
})



import os
from fbchat import log, Client
from smsgateway import sink_sms
from smsgateway.config import *

IDENTIFIER = "FB"

# Subclass fbchat.Client and override required methods
class EchoBot(Client):
    def onMessage(self, author_id, message, thread_id, thread_type, **kwargs):
        self.markAsDelivered(author_id, thread_id)
        self.markAsRead(author_id)

        log.info("Message from {} in {} ({}): {}".format(author_id, thread_id, thread_type.name, message))

        # If you're not the author, echo
        if author_id != self.uid:
            source_name = author_id
            for u in users:
                if str(u.uid) == str(author_id):
                    source_name = u.name
                    break

            print("From: %s\n\n%s\n" % (source_name, message))
            sink_sms.send(IDENTIFIER, message, source_name)
#            self.sendMessage(message, thread_id=thread_id, thread_type=thread_type)

session_cookies = None
if os.path.exists(FB_COOKIE_PATH):
    with open(FB_COOKIE_PATH, 'r') as f:
         session_cookies = f.read()

client = EchoBot(FB_CREDENTIALS[0], FB_CREDENTIALS[1], session_cookies=session_cookies)
session_cookies = client.getSession()
with open(FB_COOKIE_PATH, 'w') as f:
    f.write(str(session_cookies))
    #f.write(json.dumps(session_cookies))

users = client.fetchAllUsers()

client.listen()



#!/usr/bin/python
import subprocess, datetime, os, sys, re, json, textwrap
from smsgateway import sink_sms
from smsgateway.sources.utils import *
from smsgateway.config import *

app_log = setup_logging("telegram")

IDENTIFIER = "TG"

def listen():
    print("Starting telegram..")
    proc = subprocess.Popen([TELEGRAM_CLI_PATH, '-RW', '--json', '--disable-colors', '-k', TELEGRAM_KEY_PATH], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=sys.stdout.buffer)

    while proc.poll() == None:
        line = proc.stdout.readline().decode('UTF-8')
        app_log.debug(line)
        #proc.stdin.write(bytes('\n', 'UTF-8'))
        res = parse_message(line)
        if res:
          (m, f, to, from_id) = res
          app_log.info("From: %s\n" % f)

          if from_id != TG_USER_ID:
            app_log.info("Forwarding message:\n%s" % m)
            sink_sms.send(IDENTIFIER, m, f)
          else:
            app_log.info("Forwarding message from myself:\n%s" % m)
            sink_sms.send_from_me(IDENTIFIER, to, m)

def parse_message(line):
  if line != '':
      line_cleaned = line.strip()
      m = re.match("^>?[^{]*(\{.*\})$", line_cleaned)
      if m:
          line_cleaned = m.group(1)
          app_log.debug(line_cleaned)
          j = json.loads(line_cleaned)
          event = j['event']
          #print("Event: %s" % event)
          if event == 'message':
            try:
              f = j['from']['print_name']
              m = ''
              if 'reply_id' in j and j['reply_id']:
                app_log.info("Adding reply_to for id %s" % j['reply_id'])
                command = "get_message %s" % j['reply_id']
                try:
                  (success, res) = run_cmd([TELEGRAM_CLI_PATH, "-D", "--json", "-k", TELEGRAM_KEY_PATH, "--profile", "profile_send", "-RW", "-e", command], timeout=30)
                  if success:
                    app_log.debug("Successful get_message command")
                    for l in res.split('\n'):
                      app_log.debug("Line: %s" % l)
                      try:
                        res2 = parse_message(l)
                        if res2:
                          (m2, f2, to2, from_id) = res2
                          app_log.info("Found reply:\n%s" % m)
                          m = "Reply to %s:\n%s\n\n" % (f2, textwrap.indent(m2, '> '))
                          break
                        else:
                          app_log.warning("Reply_to: parse_message failed")
                      except Exception as e:
                        app_log.warning("Reply_to: parse_message failed:\n%s" % e)
                        pass
                  else:
                    app_log.warning("Reply_to: get_message failed:\n%s" % res)
                except Exception as e:
                  app_log.warning("Reply_to: get_message failed:\n%s" % e)

              if 'media' in j:
                media_type = j['media']['type']
                m += "Media: %s" % media_type.capitalize()
                #TODO: not working?
                if media_type == 'geo' and 'latitude' in j['media'] and 'longitude' in j['media']:
                  m += "\nhttps://osmand.net/go?lat=%s&lon=%s&z=15" %(j['media']['latitude'], j['media']['longitude'])
                elif media_type == 'contact' and ('phone' in j['media'] or 'first_name' in j['media']):
                  m += "\n"
                  m += ' '.join([x for x in j['media'][field] for field in ['first_name', 'last_name', 'phone']])
                if 'caption' in j['media'] and j['media']['caption']:
                  m += "\n%s" % j['media']['caption']
                if 'text' in j:
                  m += "\n%s" % j['text']
              elif 'text' in j:
                m += j['text']

              m = replaceEmoticons(m)
              peer_type = j['to']['peer_type']
              if peer_type == 'chat':
                # Group message
                to = j['to']['print_name'] if 'print_name' in j['to'] else j['to']['title']
                f += "@%s" % to
              elif peer_type == 'user':
                if 'username' in j['to']:
                  to_id = j['to']['username']
                to = j['to']['print_name']
              else:
                app_log.warning("Unknown peer_type: %s" % peer_type)
              from_id = j['from']['peer_id']
              return (m, f, to, from_id)
            except KeyError:
              app_log.error("KeyError in message \"%s\"" % line_cleaned)

listen()



import re, subprocess, json
from smsgateway.sources.sms import command_list
from smsgateway.config import *
from smsgateway import sink_sms

command_regex = re.compile('^(?P<command>[a-zA-Z ]+)$')

def check(cmd, multiline):
    # print("Checking %s" % cmd)
    if cmd.lower() == 'tg-old' and multiline:
      return True
    else:
      return False


def run(lines):
    print("Forwarding Telegram Message")
    toL = lines[1]
    m = re.match("To: (.*)$", toL)
    if m:
      to = m.group(1).replace(' ', '')
      print("Matched To: %s" % to)
      msg = '\n'.join(lines[2:])
      print("Sending Telegram msg:\n%s" % msg)
      arg = "msg %s %s" % (to, msg)
      print("Calling Telegram with %s" % arg)
      import getpass
      print("I am: %s" % getpass.getuser())
      try:
          #res = subprocess.check_output(["sudo", "-Hu", "pi", "/home/pi/tg/bin/telegram-cli", "-D", "--json", "-RW", "-e", arg], stderr=subprocess.STDOUT).decode('UTF-8').strip()
          res = subprocess.check_output([TELEGRAM_CLI_PATH, "-D", "--json", "-k", TELEGRAM_KEY_PATH, "--profile", "profile_send", "-RW", "-e", arg], stderr=subprocess.STDOUT, timeout=300).decode('UTF-8').strip()
      except subprocess.TimeoutExpired as e:
          return "Timeout when sending TG to %s: %s" % (to, e)
      except subprocess.CalledProcessError as e:
          return "Failed to send TG to %s: %s" % (to, e.output)
      lines = res.split('\n')
      print("Got %s" % lines)
      if len(lines) >= 1:
        for l in lines:
            try:
                j = json.loads(l.strip())
                if j and j['result'] == 'SUCCESS':
                  ret = f'TG\nTo: {to}\n{msg}'
                  break
                elif j:
                  try:
                      ret = f"Failed to send TG to {to}:\n{j['error']}"
                      break
                  except Exception as e:
                      ret = "Failed to send TG to %s!" % to
                else:
                  ret = "Failed to send TG to %s!" % to
            except ValueError:
                ret = "Failed to send TG to %s!" % to
      else:
        ret = "Failed to send TG to %s!" % to
    else:
      ret = "Couldn't match To: %s" % toL
      print(ret)
    return ret

command_list.append({
    'name' : 'TG-Forwarder',
    'check': check,
    'run': run
})



import datetime, os
from smsgateway.config import *

def send_dict(type, text, headers):
    msg = f"{type}\n"
    msg += '\n'.join([f"{k.capitalize()}: {v}" for k,v in headers.items()])
    msg += f"\n\n{text}"
    send_to(CONTROL_PHONES[0], msg)
def send(type, text, _from, phone=None, group=None):
    lines = [
        type,
        f"From: {_from}"
    ]
    if group:
        lines += [f"Group: {group}"]
    if phone:
        lines += [f"Phone: {phone}"]
    lines += [
        "",
        text
    ]
    send_to(CONTROL_PHONES[0], '\n'.join(lines))

def send_from_me(type, text, to):
    lines = [
        type,
        f"To: {to}",
        "",
        text
    ]
    send_to(CONTROL_PHONES[0], '\n'.join(lines))

def send_notif(text):
    send_to(CONTROL_PHONES[0], text)

def send_to(to, text):
    print("Sending SMS:\n%s" % text)
    header = '\n'.join([f"To: {to}", "Alphabet: UCS-2", "", ""])
    d = datetime.datetime.now().strftime("%y-%m-%d_%H-%M-%S.%f")[:-3]
    p = os.path.join(SMS_DIR, d)
    print("Writing sms to %s" % p)
    with open(p, 'wb') as f:
        f.write(header.encode("UTF-8"))
        f.write(text.encode("utf-16-be"))
        f.flush()



#!/usr/bin/python
import subprocess, datetime, os, sys, re, json
from smsgateway import sink_sms
from smsgateway.config import *
from smsgateway.sources.utils import *

app_log = setup_logging("telegram")

IDENTIFIER = "SG"

def listen():
    print("Starting signal-cli..")
    #TODO: remove -t?
    proc = subprocess.Popen([SIGNAL_CLI_PATH, '-u', SIGNAL_NUMBER, 'receive', '--json', '-t', '300'], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=sys.stdout.buffer)

    while proc.poll() == None:
        line = proc.stdout.readline().decode('UTF-8')
        app_log.debug(line)
        #proc.stdin.write(bytes('\n', 'UTF-8'))
        if line != '':
            try:
                j = json.loads(line)
                envelope = j['envelope']
                source = envelope['source']
                dataMsg = envelope['dataMessage']
                syncMsg = envelope['syncMessage']
                if dataMsg:
                    msg = dataMsg['message']
                    app_log.info("From: %s\n\n%s\n" % (source, msg))
                    sink_sms.send(IDENTIFIER, msg, source)
                elif syncMsg and source == SIGNAL_NUMBER:
                    msg = syncMsg['message']
                    app_log.info("From: %s\n\n%s\n" % (source, msg))
                    app_log.info("Message from myself: %s" % line)
                    continue
            except KeyError:
              app_log.warning("KeyError in message \"%s\"" % line_cleaned)
            # with open ('/var/log/signal.log', 'a') as f:
            #   f.write("%s\n" % line)
            app_log.info("Unknown message: %s" % line)

listen()

# {
#   "envelope": {
#     "source": "+49123123123",
#     "sourceDevice": 2,
#     "relay": null,
#     "timestamp": 1499249864225,
#     "isReceipt": false,
#     "dataMessage": {
#       "timestamp": 1499249864225,
#       "message": "abc",
#       "expiresInSeconds": 0,
#       "attachments": [],
#       "groupInfo": null
#     },
#     "syncMessage": null,
#     "callMessage": null
#   }
# }



import sys, os, signal, traceback
from smsgateway import sink_sms
from smsgateway.sources.utils import *
from smsgateway.config import *

from telethon import TelegramClient, events
from telethon.tl.types import Chat, User, Channel, \
  MessageMediaGeo, MessageMediaContact, MessageMediaPhoto, \
  MessageMediaDocument, MessageMediaWebPage, \
  Document, DocumentAttributeFilename, DocumentAttributeSticker
from telethon.tl.functions.users import GetFullUserRequest

app_log = setup_logging("telegram")

IDENTIFIER = "TG"

api_id = 242101
api_hash = "80cbc97ce425aae38c1e0291ef2ab2a4"

session_path = os.path.join(CONFIG_DIR, 'telegram')
client = TelegramClient(session_path, api_id, api_hash,update_workers=1, spawn_read_thread=False)
if not client.start():
    app_log.error("Could not connect! Run python3 -m smsgateway.sources.telegram to authorize!")
    sys.exit(1)

app_log.info("Started TelegramClient")


def parseMedia(media, msg):
    if msg:
        msg += "\n"
    if isinstance(media, MessageMediaGeo):
        geo = media.geo
        msg += f"https://osmand.net/go?lat={geo.lat}&lon={geo.long}&z=15"
    elif isinstance(media, MessageMediaPhoto):
        photo = media.photo
        msg += "Media: Photo"
        if photo.sizes:
            largest = [s for s in photo.sizes if s.type == 'y']
            if len(largest) >= 1:
                largest = largest[0]
                msg += f" ({largest.w}x{largest.h})"
    elif isinstance(media, MessageMediaContact):
        msg += "Media: Contact\n"
        if media.first_name or media.last_name:
            msg += f"Name:"
            if media.first_name:
                msg += f" {media.first_name}"
            if media.last_name:
                msg += f" {media.last_name}"
            msg += "\n"
        if media.phone_number:
            msg += f"Phone: {media.phone_number}"
    elif isinstance(media, MessageMediaDocument):
        document = media.document
        filename = None
        if document.attributes:
            filename = [attr.file_name for attr in document.attributes if isinstance(attr, DocumentAttributeFilename)]
            if len(filename) > 0:
                filename = filename[0]
        if isinstance(document, Document) and document.mime_type == "image/webp":
            msg += "Sticker"
            alt_smiley = [attr.alt for attr in document.attributes if isinstance(attr, DocumentAttributeSticker)]
            if len(alt_smiley) > 0:
                msg += f": {alt_smiley[0]}"
            msg += "\n"
        else:
          msg += "Media: "
          if document.mime_type.startswith("video"):
              if filename == "giphy.mp4":
                msg += "GIF\n"
              else:
                msg += "Video\n"
          else:
              msg += "Media: File\n"
          if filename:
            msg += f"Filename: {filename}\n"
          if document.size:
              size = sizeof_fmt(document.size)
              msg += f"Size: {size}\n"
    elif isinstance(media, MessageMediaWebPage):
        webpage = media.webpage
        msg += "Media: Webpage\n"
        msg += f"> {webpage.site_name}\n> {webpage.title}\n> {webpage.description}\n\n"
    else:
        msg += "Media: Unknown"
    return msg

def get_user_info(entity):
    info = {}
    if isinstance(entity, User):
      #user = client(GetFullUserRequest(user_id)).user
      if entity.phone:
          info['phone'] = entity.phone
      info['name'] = ' '.join([x for x in [entity.first_name, entity.last_name] if x])
      # if user_number:
      #     user_name += " (%s)" % user_number
    return info

def get_chat_info(id):
    entity = client.get_entity(id)
    info = {}
    if isinstance(entity, Chat):
        info['group'] = entity.title
    elif isinstance(entity, Channel):
        info['channel'] = entity.title
    return info
def get_channel_info(entity):
    info = {}
    return info

@client.on(events.NewMessage())
@client.on(events.MessageEdited())
def callback(event):
    chat_info = {'ID': event.message.id}
    try:
      if event.message.out or not event.message.from_id:
        user_id = event.message.to_id
      else:
        user_id = event.message.from_id


      user_entity = client.get_entity(user_id)
      user_info = get_user_info(user_entity)
      if 'name' in user_info:
        chat_info['to' if event.message.out else 'from'] = user_info['name']
      if 'phone' in user_info:
        chat_info['phone'] = user_info['phone']

      group_info = get_chat_info(event.message.to_id)
      if group_info:
        chat_info.update(group_info)
        # if event.message.out: # if sent to group/channel, get name
        # for key in ['group', 'channel']:
        #   if key in group_info:
        #       chat_info['group'] = group_info[key]
        #       if event.message.out:
        #           chat_info['to'] = group_info[key]
        # else: # from someone else
        #     chat_info['from'] =
      # if isinstance(group_entity, Chat):
      #     chat_info['group'] = group_entity.title
      #     if event.message.out:
      #         user_name = chat_info['group']
      # elif isinstance(group_entity, Channel):
      #     chat_info['channel'] = group_entity.title
      #     if event.message.out:
      #         user_name = chat_info['channel']
      if event.message.edit_date:
          chat_info['edit'] = "True"
      msg = ""

      if event.message.fwd_from:
          fwd_from = event.message.fwd_from
          if fwd_from.from_id:
              user_info = get_user_info(from_id)
          elif fwd_from.channel_id:
              user_info = get_chat_info(fwd_from.channel_id)
          msg += f"Forwarded from {user_info['name']}:\n"
          # if 'name' in user_info:
          #   chat_info['from_name'] = user_info['name']
          # if 'phone' in user_info:
          #   chat_info['from_phone'] = user_info['phone']

      # if event.message.reply_to_msg_id:
      #     for message in client.iter_messages('username', limit=10):
      msg += event.message.message

      media = event.message.media
      if media:
          msg = parseMedia(media, msg)

      if event.message.out:
          for key in ['to', 'group', 'channel']:
            if key in chat_info:
              app_log.info("New message to %s!" % chat_info[key])
              break
          sink_sms.send_dict(IDENTIFIER, msg, chat_info)
          # sink_sms.send_from_me(IDENTIFIER, msg, chat_info['to'])
      else:
          for key in ['from', 'group', 'channel']:
            if key in chat_info:
              app_log.info("New message from %s!" % chat_info[key])
              break
          sink_sms.send_dict(IDENTIFIER, msg, chat_info)
          # sink_sms.send(IDENTIFIER, msg, user_name, user_number, group_name)
      app_log.debug(msg)
    except Exception as e:
        app_log.warning(traceback.format_exc())
        app_log.warning(event.stringify())
        app_log.warning(str(chat_info))

app_log.info("Catching up..")

def handler(signum, frame):
    if signum == signal.SIGALRM:
      app_log.info("Got a signal for timeout, raising exception")
      raise Exception("Function timed out!")
signal.signal(signal.SIGALRM, handler)
signal.alarm(10)
try:
    client.catch_up()
except Exception as e:
    app_log.warning("client.catch_up failed with Exception: %s" % e)
signal.alarm(0)

app_log.info("Listening to messages..")

client.idle()
